from __future__ import annotations

import importlib
import unittest

tool_view = importlib.import_module("core.tool_view")


class ToolViewNavigateTests(unittest.TestCase):
    def test_navigate_search_text_shows_query_and_filters(self) -> None:
        text = tool_view.format_tool_view_text(
            "navigate",
            {
                "search": "entari 0.17",
                "df": "2026-03-13..2026-03-26",
                "count": 5,
            },
            max_chars=300,
        )

        self.assertEqual(
            text,
            'Navigate Search "entari 0.17" [df=2026-03-13..2026-03-26] · 5 lines',
        )

    def test_navigate_text_shows_title_target_and_stats(self) -> None:
        text = tool_view.format_tool_view_text(
            "navigate",
            {
                "url": "https://www.netflix.com/tw/title/81756595",
                "title": "Watch Cosmic Princess Kaguya! | Netflix Official Site",
                "count": 30,
                "total_lines": 266,
            },
            max_chars=300,
        )

        self.assertEqual(
            text,
            'Navigate 30 lines "Watch Cosmic Princess Kaguya! | Netflix Official Site"',
        )

    def test_navigate_text_shows_keep_lines(self) -> None:
        text = tool_view.format_tool_view_text(
            "navigate",
            {
                "ref": "44:2",
                "keep": ["L120-L180", "L220"],
            },
            max_chars=300,
        )

        self.assertEqual(
            text,
            'Navigate in "#44:2" keep L120-L180, L220',
        )


if __name__ == "__main__":
    unittest.main()



from __future__ import annotations

from copy import deepcopy
from typing import Any

BASE_SYSTEM_PROMPT = """\
# 你的身份
- You are {name}, 你是 `HYW` 的快速响应 `搜索引擎代理`, 一旦证据足够，就停止新增搜索并转入整合详细回答.
- Current time: {time}。这是重要信息，搜索时必须严格参考此时间作为有效性评估!!!!!!
- 面向用户的 Preferred language: {language}
{custom}

## 工具
- `navigate(url|ref|search, keep=[...])`
    - 匿名打开网页(Markdown)，查询最新资料.
    - 用 `navigate(search=...)` 发起搜索, 而不是自己拼写搜索引擎；
    - 搜索语言与强搜索词约束：仅根据搜索目标本身的客观来源语言(必须遵守!!!!!!)决定，严格排除用户提问语言和偶然搜索结果语言的干扰。
    - `search` 精准短小, 去除助词, 不要把全场景全塞进一条 query, 必须收缩到最核心的 1-2 个实体、正式名称、核心别名或强识别短语。
    - 需要地区、时间过滤时: 填写可选参数: `kl`(地区), `df`(例如 df="2026-03-13..2026-03-26")。
    - 页面中的所有链接会被压缩为页面内 ref，例如 `4:2` 表示 page 4 的第 2 个链接；可继续使用 `navigate(ref="4:2")`。
    - 每一次 `navigate` 都必须显式填写 `keep`，这是强制参数；如果这次不需要保留旧页内容，也必须写 `keep="L0"`。
    - 若当前已有打开页，而你想在继续 `navigate` 前保留旧页证据，就把那些精确可见行写进这次 `navigate` 的 `keep`；它会先保留当前打开页里的精确可见行，再替换到新页面。
    - `keep` 积极填写当前打开页里真实可见的精确行号或行范围，例如 `L1`、`L12-L18`, 慢慢收集长期记忆.
    - 不导航百度贴吧、抖音、淘宝等强需求登陆、电商广告等页面.

## 每次必须满足轮次规则 [重要]
- 汇总各种资料, 不要被单一结果带偏, `搜索引擎代理` 不需要评估信息是否 100% 官方.
- 使用合适的工具收集情报、改善搜索词与搜索策略, 直到定位到关键信息, 有证据就答!!!
- 调用工具前先简单使用无加粗的普通文本, 1-3 句话简单汇报, 再同时进行其他操作.
- 最终答复: 额外包含相关信息介绍，至少覆盖“结论 + 判断依据 + 相关背景”, 适度润色表达，让答案更顺滑，但不要脱离证据自由发挥。
"""

FIRST_SEARCH_PROMPT = """
## 优先搜索确认信息, 因为 Current time 差异, 严格禁止凭借记忆捏造关键词
- 首轮搜索严格参考 `search` 编写规范以及 用户消息: {user_message} , 不带任何补充防止因幻觉跑偏
- 若用户给了链接，优先 `navigate(url=..., keep="L0")` 直接读取页面。
- 默认认为任何孤立名词、短语、模型 ID、包名、仓库名、产品名都默认视为待查询对象，禁止把它理解成用户在叫你、给你命名或要求你自我介绍。

## 包含图片
不搜索立即总结图片内容(较完善描述), 给出几个可能的用户任务方向, 等待下一次回复.

## 拒绝
- 违纪违法、色情见证内容
"""
ACTIVE_PAGE_STATE_PROMPT = "You have {count} active page(s): [{ids}]"

POST_SEARCH_PROMPT = ""

LATE_ROUND_FINAL_REPLY_PROMPT = "已经濒临结束, 请立即整理现有消息做阶段性最终回复!!!"
SEARCH_RESULT_REMINDER = "- 搜索结果页与普通页面都通过 `navigate(...)` 暴露；若要继续深入，直接用 `navigate(ref=...)` 跟进对应结果。"

DEFAULT_NO_TITLE_TEXT = "No Title"
DEFAULT_NO_RESULTS_TEXT = "No results."
DEFAULT_PAGE_NO_MATCHING_TEXT = "No matching lines found."
DEFAULT_PAGE_NO_MATCHING_CACHED_TEXT = "No matching lines found in cached page content."
DUPLICATE_QUERY_SKIPPED_TEXT = "Duplicate query skipped in this session. Choose a different query or use navigate(ref=...) on an existing result."
PAGE_MARKDOWN_EMPTY_TITLE = "# Page"
PAGE_MARKDOWN_TITLE_PREFIX = "Title: "
PAGE_MARKDOWN_SCOPE_PREFIX = "Scope: "
PAGE_MARKDOWN_CACHE_PREFIX = "Cache: "
PAGE_MARKDOWN_CACHE_HIT_TEXT = "hit"
PAGE_MARKDOWN_CACHE_MISS_TEXT = "miss"
PAGE_MARKDOWN_MATCHED_LINES_TEXT = "Matched lines:"

LATEST_RAW_HEADING = "# Latest Round Raw"
LATEST_RAW_EMPTY_TEXT = "No latest raw items yet."

_TOOL_DEFINITIONS: tuple[dict[str, Any], ...] = (
    {
        "type": "function",
        "function": {
            "name": "navigate",
            "description": "Navigate to a page by url or ref, or to a search result document by search query. keep is required on every call; use keep='L0' when nothing should be preserved.",
            "parameters": {
                "type": "object",
                "properties": {
                    "url": {
                        "type": "string",
                        "description": "Direct page URL. Omit when using ref.",
                    },
                    "ref": {
                        "type": "string",
                        "description": "Use an existing id or page ref, for example '3' or '4:2'. '4:2' means page 4, link 2.",
                    },
                    "search": {
                        "type": "string",
                        "description": "Navigate to a search result document for this query instead of opening a page url.",
                    },
                    "kl": {
                        "type": "string",
                        "description": "Optional region filter when using search.",
                    },
                    "df": {
                        "type": "string",
                        "description": "Optional time filter when using search.",
                    },
                    "keep": {
                        "description": "Required. Exact visible lines or line ranges to keep from the current opened page before this navigate replaces it. Use 'L0' when nothing should be preserved.",
                        "oneOf": [
                            {"type": "string"},
                            {
                                "type": "array",
                                "items": {"type": "string"},
                            },
                        ],
                    },
                },
                "required": ["keep"],
                "additionalProperties": False,
            },
        },
    },
)


def litellm_tools_for_phase(phase: str, *, disclosure_step: int | None = None) -> list[dict[str, Any]]:
    del phase, disclosure_step
    tools: list[dict[str, Any]] = []
    for item in _TOOL_DEFINITIONS:
        tools.append(deepcopy(item))
    return tools


def litellm_tool_config_for_phase(phase: str, *, disclosure_step: int | None = None) -> dict[str, Any]:
    return {
        "tools": litellm_tools_for_phase(phase, disclosure_step=disclosure_step),
        "tool_choice": "auto",
        "parallel_tool_calls": False,
    }



from __future__ import annotations

from typing import Any, Dict

from .jina_common import (
    build_usage_meta,
    has_auth_header,
    jina_headers,
    load_httpx_module,
)


def _jina_headers(
    *,
    config: dict[str, Any] | None = None,
    capability: str,
    default_accept: str,
) -> dict[str, str]:
    default_headers = {}
    if capability == "page_extract":
        default_headers = {
            "X-Engine": "browser",
            "X-Return-Format": "markdown",
        }
    return jina_headers(
        config=config,
        provider_name="jina_ai",
        capability=capability,
        default_accept=default_accept,
        default_headers=default_headers,
    )


def _normalize_reader_source_url(url: str) -> str:
    text = str(url or "").strip()
    if not text:
        raise ValueError("url is empty")
    if not text.startswith(("http://", "https://")):
        text = "https://" + text
    return text


def _coerce_text(value: Any) -> str:
    return str(value or "").strip()


def _parse_jina_reader_text(text: str, fallback_url: str) -> dict[str, Any]:
    title = ""
    source_url = fallback_url
    body_lines: list[str] = []
    in_body = False

    for raw_line in str(text or "").splitlines():
        line = raw_line.rstrip()
        if not in_body and line.startswith("Title:"):
            title = line.split(":", 1)[1].strip()
            continue
        if not in_body and line.startswith("URL Source:"):
            source_url = line.split(":", 1)[1].strip() or fallback_url
            continue
        if line.strip() == "Markdown Content:":
            in_body = True
            continue
        if in_body:
            body_lines.append(line)

    content = "\n".join(body_lines).strip() or str(text or "").strip()
    return {
        "ok": bool(content),
        "provider": "jina_ai",
        "title": title,
        "url": source_url,
        "content": content,
        "html": "",
    }


def _parse_jina_reader_json(payload: Any, fallback_url: str) -> dict[str, Any]:
    if isinstance(payload, list):
        for item in payload:
            parsed = _parse_jina_reader_json(item, fallback_url)
            if parsed.get("content"):
                return parsed
        return {
            "ok": False,
            "provider": "jina_ai",
            "title": "",
            "url": fallback_url,
            "content": "",
            "html": "",
        }

    if not isinstance(payload, dict):
        return _parse_jina_reader_text(str(payload or ""), fallback_url)

    title = _coerce_text(payload.get("title") or payload.get("name") or payload.get("headline"))
    source_url = _coerce_text(
        payload.get("url")
        or payload.get("source_url")
        or payload.get("source")
        or payload.get("href")
    ) or fallback_url
    content = _coerce_text(
        payload.get("content")
        or payload.get("text")
        or payload.get("markdown")
        or payload.get("body")
        or payload.get("md")
    )
    html = _coerce_text(payload.get("html"))

    if not content and isinstance(payload.get("data"), str):
        content = _coerce_text(payload.get("data"))

    if content or html:
        return {
            "ok": bool(content or html),
            "provider": "jina_ai",
            "title": title,
            "url": source_url,
            "content": content or html,
            "html": html,
        }

    for key in ("data", "result", "results", "item"):
        nested = payload.get(key)
        if isinstance(nested, (dict, list)):
            parsed = _parse_jina_reader_json(nested, source_url)
            if parsed.get("content"):
                if not parsed.get("title") and title:
                    parsed["title"] = title
                if not parsed.get("url"):
                    parsed["url"] = source_url
                return parsed

    for value in payload.values():
        if isinstance(value, (dict, list)):
            parsed = _parse_jina_reader_json(value, source_url)
            if parsed.get("content"):
                if not parsed.get("title") and title:
                    parsed["title"] = title
                if not parsed.get("url"):
                    parsed["url"] = source_url
                return parsed

    return {
        "ok": False,
        "provider": "jina_ai",
        "title": title,
        "url": source_url,
        "content": "",
        "html": html,
    }


async def jina_ai_page_extract(
    url: str,
    max_chars: int = 8000,
    timeout: float = 20.0,
    config: dict[str, Any] | None = None,
    **_: Any,
) -> Dict[str, Any]:
    httpx = load_httpx_module()
    source_url = _normalize_reader_source_url(url)
    target_url = f"https://r.jina.ai/{source_url}"
    headers = _jina_headers(
        config=config,
        capability="page_extract",
        default_accept="text/plain",
    )

    async with httpx.AsyncClient(timeout=timeout, follow_redirects=True) as client:
        response = await client.get(target_url, headers=headers)
        response.raise_for_status()
        raw_text = response.text
        meta = await build_usage_meta(
            client,
            response=response,
            raw_text=raw_text,
            provider="jina_ai",
            capability="page_extract",
            billed=has_auth_header(headers),
        )

    content_type = _coerce_text(response.headers.get("content-type")).lower()
    if "json" in content_type:
        payload = _parse_jina_reader_json(response.json(), source_url)
    else:
        try:
            payload = _parse_jina_reader_json(response.json(), source_url)
        except Exception:
            payload = _parse_jina_reader_text(response.text, source_url)
    payload["content"] = str(payload.get("content") or "")[: max(1, int(max_chars))]
    payload["_meta"] = meta
    payload["_model_markdown"] = str(payload.get("content") or "").strip()
    return payload


__all__ = ["jina_ai_page_extract"]



from __future__ import annotations

import importlib
import unittest

from rich.markdown import Markdown

cli = importlib.import_module("core.cli")


class CliRenderTests(unittest.TestCase):
    def test_stream_preview_renders_markdown_immediately(self) -> None:
        rendered = cli._render_stream_preview("# 标题\n\n- 项目")

        self.assertIsInstance(rendered, Markdown)

    def test_stabilize_stream_markdown_closes_unfinished_code_fence(self) -> None:
        stabilized = cli._stabilize_stream_markdown("```python\nprint('x')")

        self.assertTrue(stabilized.rstrip().endswith("```"))


if __name__ == "__main__":
    unittest.main()



"""Thin package entrypoint that re-exports the core entari plugin implementation."""

from core import entari_plugin_hyw as _impl
from core.entari_plugin_hyw import *  # noqa: F401,F403

__version__ = _impl.__version__
__plugin__ = _impl.__plugin__



from __future__ import annotations

import importlib
import unittest

config = importlib.import_module("core.config")


class ModelConfigServiceTierTests(unittest.TestCase):
    def test_build_model_config_keeps_service_tier_from_active_profile(self) -> None:
        cfg = config.build_model_config(
            {
                "models": [
                    {
                        "name": "codex-fast",
                        "model": "gpt-5.4",
                        "model_provider": "mirror",
                        "service_tier": "fast",
                    },
                    {
                        "name": "fallback",
                        "model": "openrouter/openai/gpt-5.4-nano",
                    },
                ]
            }
        )

        self.assertEqual(cfg.get("model"), "gpt-5.4")
        self.assertEqual(cfg.get("service_tier"), "fast")


if __name__ == "__main__":
    unittest.main()



from __future__ import annotations

import importlib
from types import SimpleNamespace
import unittest
from unittest import mock

search_ddgs = importlib.import_module("core.search_ddgs")
search_jina_ai = importlib.import_module("core.search_jina_ai")
web_runtime = importlib.import_module("core.web_runtime")


class _FakeResponse:
    def __init__(self, *, payload: dict[str, object] | None = None, text: str = ""):
        self._payload = payload if isinstance(payload, dict) else {}
        self.text = text
        self.headers: dict[str, str] = {}

    def raise_for_status(self) -> None:
        return None

    def json(self) -> dict[str, object]:
        if self._payload:
            return self._payload
        raise ValueError("no json payload")


class _FakeAsyncClient:
    def __init__(self, recorder: dict[str, object], response: _FakeResponse, *args, **kwargs):
        del args, kwargs
        self._recorder = recorder
        self._response = response

    async def __aenter__(self) -> "_FakeAsyncClient":
        return self

    async def __aexit__(self, exc_type, exc, tb) -> bool:
        del exc_type, exc, tb
        return False

    async def get(self, url: str, *, params=None, headers=None):
        self._recorder["url"] = url
        self._recorder["params"] = params
        self._recorder["headers"] = dict(headers or {})
        return self._response


class SearchProviderTests(unittest.IsolatedAsyncioTestCase):
    def test_build_duckduckgo_html_url_keeps_region_and_time_filters(self) -> None:
        url = search_ddgs.build_duckduckgo_html_url(
            "claudecode",
            kl="us-en",
            df="2026-03-13..2026-03-26",
            t="h_",
            ia="web",
        )

        self.assertEqual(
            url,
            "https://html.duckduckgo.com/html/?q=claudecode&l=us-en&df=2026-03-13..2026-03-26&t=h_&ia=web",
        )

    async def test_jina_ddgs_search_wraps_duckduckgo_html_url(self) -> None:
        recorder: dict[str, object] = {}
        response = _FakeResponse(
            text=(
                "# claudecode at DuckDuckGo\n\n"
                "## [Claude Code Ultimate Guide - GitHub]"
                "(https://duckduckgo.com/l/?uddg=https%3A%2F%2Fgithub.com%2Fdemo%2Frepo&rut=test)\n\n"
                "[github.com/demo/repo]"
                "(https://duckduckgo.com/l/?uddg=https%3A%2F%2Fgithub.com%2Fdemo%2Frepo&rut=test)\n\n"
                "[Guide summary line]"
                "(https://duckduckgo.com/l/?uddg=https%3A%2F%2Fgithub.com%2Fdemo%2Frepo&rut=test)\n"
            )
        )

        class _FakeHttpxModule:
            def AsyncClient(self, *args, **kwargs):
                return _FakeAsyncClient(recorder, response, *args, **kwargs)

        async def _fake_usage_meta(*args, **kwargs):
            del args, kwargs
            return {}

        with (
            mock.patch.object(search_ddgs, "load_httpx_module", return_value=_FakeHttpxModule()),
            mock.patch.object(search_ddgs, "build_usage_meta", side_effect=_fake_usage_meta),
        ):
            payload = await search_ddgs.jina_ddgs_search(
                "claudecode",
                kl="us-en",
                df="2026-03-13..2026-03-26",
                t="h_",
                ia="web",
            )

        self.assertEqual(
            recorder.get("url"),
            "https://r.jina.ai/https://html.duckduckgo.com/html/?q=claudecode&l=us-en&df=2026-03-13..2026-03-26&t=h_&ia=web",
        )
        self.assertEqual(payload["results"][0]["title"], "Claude Code Ultimate Guide - GitHub")
        self.assertEqual(payload["results"][0]["url"], "https://github.com/demo/repo")

    def test_jina_page_extract_keeps_authorization_when_configured(self) -> None:
        headers = search_jina_ai._jina_headers(
            config={
                "tools": {
                    "config": {
                        "jina_ai": {
                            "page_extract": {
                                "headers": {
                                    "Authorization": "Bearer test-token",
                                }
                            }
                        }
                    }
                }
            },
            capability="page_extract",
            default_accept="text/plain",
        )

        self.assertEqual(headers.get("Authorization"), "Bearer test-token")
        self.assertEqual(headers.get("X-Engine"), "browser")


class SearchFallbackTests(unittest.IsolatedAsyncioTestCase):
    async def test_search_falls_back_to_ddgs_on_primary_connect_error(self) -> None:
        async def _raise_connect_error(**kwargs):
            del kwargs
            raise RuntimeError("ConnectError")

        async def _return_ddgs_result(**kwargs):
            del kwargs
            return [
                {
                    "title": "Fallback Result",
                    "url": "https://example.com/fallback",
                    "snippet": "ok",
                }
            ]

        jina_ddgs_handler = SimpleNamespace(provider="jina_ddgs", target="jina_ddgs", callable=_raise_connect_error)
        ddgs_handler = SimpleNamespace(provider="ddgs", target="ddgs", callable=_return_ddgs_result)

        def _fake_resolve(config, capability, selection=None):
            del config
            if capability == "search" and selection is None:
                return [jina_ddgs_handler]
            if capability == "search" and selection == "ddgs":
                return [ddgs_handler]
            return []

        with mock.patch.object(web_runtime, "resolve_tool_handlers", side_effect=_fake_resolve):
            suite = web_runtime.WebToolSuite(config={"tools": {"use": {"search": "jina_ddgs"}}})
            rows, meta = await suite._search_text("葱喵Bot", None, None, None, None, 5)

        self.assertEqual(meta, {})
        self.assertEqual(rows[0]["provider"], "ddgs")
        self.assertEqual(rows[0]["url"], "https://example.com/fallback")

    async def test_search_falls_back_to_ddgs_on_primary_timeout(self) -> None:
        async def _sleep_forever_enough(**kwargs):
            del kwargs
            await __import__("asyncio").sleep(0.05)
            return []

        async def _return_ddgs_result(**kwargs):
            del kwargs
            return [
                {
                    "title": "Fallback Result",
                    "url": "https://example.com/fallback-timeout",
                    "snippet": "ok",
                }
            ]

        jina_ddgs_handler = SimpleNamespace(provider="jina_ddgs", target="jina_ddgs", callable=_sleep_forever_enough)
        ddgs_handler = SimpleNamespace(provider="ddgs", target="ddgs", callable=_return_ddgs_result)

        def _fake_resolve(config, capability, selection=None):
            del config
            if capability == "search" and selection is None:
                return [jina_ddgs_handler]
            if capability == "search" and selection == "ddgs":
                return [ddgs_handler]
            return []

        with (
            mock.patch.object(web_runtime, "resolve_tool_handlers", side_effect=_fake_resolve),
            mock.patch.object(web_runtime, "_resolve_search_handler_timeout_s", return_value=0.01),
        ):
            suite = web_runtime.WebToolSuite(config={"tools": {"use": {"search": "jina_ddgs"}}})
            rows, meta = await suite._search_text("葱喵Bot", None, None, None, None, 5)

        self.assertEqual(meta, {})
        self.assertEqual(rows[0]["provider"], "ddgs")
        self.assertEqual(rows[0]["url"], "https://example.com/fallback-timeout")


if __name__ == "__main__":
    unittest.main()



from __future__ import annotations

import json
import re
from typing import Any

FILTER_KEYS: tuple[str, ...] = ("kl", "df")


def coerce_search_filters(payload: dict[str, Any] | None) -> dict[str, str]:
    data = payload if isinstance(payload, dict) else {}
    filters: dict[str, str] = {}

    region = str(data.get("kl") or data.get("region") or "").strip().lower()
    if region:
        filters["kl"] = region

    date_filter = str(data.get("df") or data.get("timelimit") or "").strip()
    if date_filter:
        filters["df"] = date_filter

    return filters


def format_search_filters(filters: dict[str, Any] | None) -> str:
    data = coerce_search_filters(filters)
    parts: list[str] = []
    for key in FILTER_KEYS:
        value = str(data.get(key) or "").strip()
        if value:
            parts.append(f"{key}={value}")
    return ", ".join(parts)


def format_search_request_label(query: str, filters: dict[str, Any] | None = None) -> str:
    clean_query = str(query or "").strip()
    filter_text = format_search_filters(filters)
    if clean_query and filter_text:
        return f"{clean_query} [{filter_text}]"
    return clean_query


def normalize_search_request_key(query: str, filters: dict[str, Any] | None = None) -> str:
    clean_query = re.sub(r"\s+", " ", str(query or "").strip().strip("`'\"")).lower()
    if not clean_query:
        return ""
    normalized_filters = coerce_search_filters(filters)
    if not normalized_filters:
        return clean_query
    return clean_query + "\n" + json.dumps(normalized_filters, ensure_ascii=False, sort_keys=True)


def build_search_result_lines(
    public_results: list[dict[str, Any]],
    *,
    default_no_title_text: str = "No Title",
    snippet_limit: int = 180,
) -> list[str]:
    rows = [item for item in public_results if isinstance(item, dict)]
    if not rows:
        return []
    width = max(2, len(str(len(rows))))
    rendered: list[str] = []
    for index, item in enumerate(rows, start=1):
        title = str(item.get("title") or default_no_title_text).strip() or default_no_title_text
        domain = str(item.get("domain") or "").strip()
        snippet = str(item.get("snippet") or item.get("intro") or "").strip()
        snippet = re.sub(r"\s+", " ", snippet).strip()
        if len(snippet) > max(1, int(snippet_limit)):
            snippet = snippet[: max(1, int(snippet_limit)) - 1].rstrip() + "…"
        parts = [f"[{index}] {title}"]
        if domain:
            parts.append(domain)
        matched_queries = [
            str(item_query or "").strip()
            for item_query in item.get("matched_queries", [])
            if str(item_query or "").strip()
        ]
        if matched_queries:
            parts.append("matched=" + " | ".join(matched_queries[:3]))
        if snippet:
            parts.append(snippet)
        rendered.append(f"L{index:0{width}d} | " + " | ".join(parts))
    return rendered


def build_search_open_lines(
    public_results: list[dict[str, Any]],
    *,
    default_no_title_text: str = "No Title",
    snippet_limit: int = 180,
) -> list[str]:
    rows = [item for item in public_results if isinstance(item, dict)]
    rendered: list[str] = []
    for index, item in enumerate(rows, start=1):
        title = str(item.get("title") or default_no_title_text).strip() or default_no_title_text
        url = str(item.get("url") or "").strip()
        domain = str(item.get("domain") or "").strip()
        snippet = str(item.get("snippet") or item.get("intro") or "").strip()
        snippet = re.sub(r"\s+", " ", snippet).strip()
        if len(snippet) > max(1, int(snippet_limit)):
            snippet = snippet[: max(1, int(snippet_limit)) - 1].rstrip() + "…"
        linked_title = f"[{title}]({url})" if url else title
        parts = [f"[{index}] {linked_title}"]
        if domain:
            parts.append(domain)
        matched_queries = [
            str(item_query or "").strip()
            for item_query in item.get("matched_queries", [])
            if str(item_query or "").strip()
        ]
        if matched_queries:
            parts.append("matched=" + " | ".join(matched_queries[:3]))
        if snippet:
            parts.append(snippet)
        rendered.append(" | ".join(parts))
    return rendered


def build_search_document_markdown(
    *,
    query: str,
    public_results: list[dict[str, Any]],
    skipped_duplicate: bool,
    reminder: str,
    error: str = "",
    search_filters: dict[str, Any] | None = None,
    direction_brief: str = "",
    expanded_queries: list[str] | None = None,
    duplicate_queries: list[str] | None = None,
    summary_text: str = "",
    default_no_results_text: str = "No results.",
    duplicate_query_skipped_text: str = "Duplicate query skipped in this session.",
    default_no_title_text: str = "No Title",
) -> str:
    lines = [f"# Search: {query}", ""]
    filter_text = format_search_filters(search_filters)
    if filter_text:
        lines.append(f"Filters: {filter_text}")
    clean_direction = str(direction_brief or "").strip()
    if clean_direction:
        lines.append(f"Direction: {clean_direction}")
    clean_queries = [str(item or "").strip() for item in expanded_queries or [] if str(item or "").strip()]
    if clean_queries:
        lines.append("Executed queries: " + ", ".join(clean_queries))
    clean_duplicates = [str(item or "").strip() for item in duplicate_queries or [] if str(item or "").strip()]
    if clean_duplicates:
        lines.append("Skipped duplicate queries: " + ", ".join(clean_duplicates))
    clean_summary = str(summary_text or "").strip()
    if clean_summary:
        lines.append(f"Search summary: {clean_summary}")

    if len(lines) > 2:
        lines.append("")

    if skipped_duplicate and not public_results and not error:
        lines.append(duplicate_query_skipped_text)
    elif error:
        lines.append(error)
    elif not public_results:
        lines.append(default_no_results_text)
    else:
        result_lines = build_search_result_lines(
            public_results,
            default_no_title_text=default_no_title_text,
        )
        lines.append(f"Preview: results={len(result_lines)}")
        lines.extend(["", "Result lines:"])
        lines.extend(result_lines)

    if reminder:
        if lines and str(lines[-1]).strip():
            lines.append("")
        lines.append(str(reminder).strip())
    return "\n".join(lines).strip()


__all__ = [
    "build_search_document_markdown",
    "build_search_open_lines",
    "build_search_result_lines",
    "coerce_search_filters",
    "format_search_filters",
    "format_search_request_label",
    "normalize_search_request_key",
]



#!/usr/bin/env python3
from __future__ import annotations

import argparse
import json
import re
from dataclasses import dataclass
from pathlib import Path
from urllib.parse import unquote, urlparse


JSON_BLOCK_RE = re.compile(r"```json\n(.*?)\n```", re.DOTALL)
PAGE_LINE_RE = re.compile(r"^L?(?P<line>\d+)\s*\|\s*(?P<text>.+)$")

TITLE_SPLIT_RE = re.compile(r"\s+-\s+|\s+\|\s+|\s+｜\s+|[_｜|·•]+")
LATIN_WORD_RE = re.compile(r"[A-Za-z][A-Za-z0-9_'-]*")
CJK_CHAR_RE = re.compile(r"[\u4e00-\u9fff\u3040-\u30ff]")
SIGNAL_CHAR_RE = re.compile(r"[\w\u4e00-\u9fff\u3040-\u30ff]")
MARKDOWN_BULLET_RE = re.compile(r"^\s*(?:[*+-]|\d+\.)\s+")
MARKDOWN_HEADING_RE = re.compile(r"^\s*#{1,6}\s+")


@dataclass
class PageEntry:
    source_log: Path
    url: str
    host: str
    raw_title: str
    title_core: str
    path_label: str
    mode: str
    matched_lines: list[tuple[int, str]]
    skeleton_lines: list[str]


def _normalize_space(text: str) -> str:
    return re.sub(r"\s+", " ", str(text or "").strip())


def _iter_json_blocks(text: str) -> list[dict]:
    rows: list[dict] = []
    for match in JSON_BLOCK_RE.finditer(text):
        block = str(match.group(1) or "").strip()
        if not block:
            continue
        try:
            payload = json.loads(block)
        except Exception:
            continue
        if isinstance(payload, dict):
            rows.append(payload)
    return rows


def _parse_page_markdown(payload: dict, *, source_log: Path) -> PageEntry | None:
    markdown = str(payload.get("_model_markdown") or "").strip()
    if not markdown.startswith("# Page:"):
        return None
    provider = str(
        payload.get("provider")
        or ((payload.get("_meta") or {}).get("provider") if isinstance(payload.get("_meta"), dict) else "")
        or ""
    ).strip()
    if provider and provider != "jina_ai":
        return None

    lines = markdown.splitlines()
    url = str(lines[0].removeprefix("# Page:").strip())
    if not url:
        return None
    raw_title = ""
    mode = "preview" if any(line.startswith("Preview: ") for line in lines) else "page"
    matched_lines: list[tuple[int, str]] = []
    for line in lines:
        if line.startswith("Title: "):
            raw_title = str(line.removeprefix("Title: ").strip())
            continue
        match = PAGE_LINE_RE.match(line)
        if match is None:
            continue
        matched_lines.append((int(match.group("line")), match.group("text")))
    if not matched_lines:
        return None

    host = urlparse(url).netloc.strip().lower()
    title_core = _title_core(raw_title, url=url)
    path_label = _path_label(url)
    skeleton_lines = _build_skeleton_lines(matched_lines)
    if not skeleton_lines:
        return None
    return PageEntry(
        source_log=source_log,
        url=url,
        host=host,
        raw_title=raw_title,
        title_core=title_core,
        path_label=path_label,
        mode=mode,
        matched_lines=matched_lines,
        skeleton_lines=skeleton_lines,
    )


def _path_label(url: str) -> str:
    parsed = urlparse(url)
    path = unquote(parsed.path or "").strip("/")
    if not path:
        return parsed.netloc or url
    parts = [part for part in path.split("/") if part]
    tail = "/".join(parts[-2:]) if len(parts) >= 2 else parts[-1]
    return tail or parsed.netloc or url


def _title_core(title: str, *, url: str) -> str:
    clean = _normalize_space(title)
    if not clean:
        return _path_label(url)

    candidates = [clean]
    for part in TITLE_SPLIT_RE.split(clean):
        part = _normalize_space(part)
        if part:
            candidates.append(part)
            for sep in (":", "："):
                if sep in part:
                    head = _normalize_space(part.split(sep, 1)[0])
                    if 2 <= len(head) <= 80:
                        candidates.append(head)

    path_tokens = {
        token.casefold()
        for token in re.split(r"[/_.-]+", _path_label(url))
        if token.strip()
    }
    best = clean
    best_score: tuple[int, int, int, int, str] | None = None
    for item in candidates:
        low = item.casefold()
        overlap = sum(1 for token in path_tokens if token and token in low)
        length_penalty = 0
        if len(item) > 90:
            length_penalty += 2
        elif len(item) > 50:
            length_penalty += 1
        if len(item) < 3:
            length_penalty += 2
        signal_chars = len(re.findall(r"[\w\u4e00-\u9fff\u3040-\u30ff]", item))
        punctuation_penalty = len(re.findall(r"[/|_·•\-:：]", item))
        score = (length_penalty, -overlap, -signal_chars, punctuation_penalty, item)
        if best_score is None or score < best_score:
            best_score = score
            best = item
    return best


def _strip_markdown_links(text: str) -> str:
    value = str(text or "")
    prev = None
    while prev != value:
        prev = value
        value = re.sub(
            r"\[!\[([^\]]*)\]\([^)]+\)\]\([^)]+\)",
            lambda match: f"[IMG:{_normalize_space(match.group(1) or 'image')}]",
            value,
        )
        value = re.sub(
            r"!\[([^\]]*)\]\([^)]+\)",
            lambda match: f"[IMG:{_normalize_space(match.group(1) or 'image')}]",
            value,
        )
        value = re.sub(
            r"\[([^\]]+)\]\([^)]+\)",
            lambda match: _normalize_space(match.group(1)),
            value,
        )
    value = re.sub(r"\[\]\([^)]+\)", "", value)
    value = re.sub(r"<https?://[^>]+>", "", value)
    value = re.sub(r"\b(?:https?://|blob:http://|mailto:|javascript:)\S+", "", value)
    value = re.sub(r"\[\[(\d+(?:[-,]\d+)*)\]\]\(?", "", value)
    value = re.sub(r"\[(\d+(?:[-,]\d+)*)\]", "", value)
    value = re.sub(r"(^|\s)>+\s*", r"\1", value)
    value = re.sub(r"(^|\s)[*_~]{1,3}(?=\S)", r"\1", value)
    value = re.sub(r"(?<=\S)[*_~]{1,3}(?=\s|$)", "", value)
    value = re.sub(r"\[[xX ]\]", "", value)
    value = value.replace("[](", "")
    value = value.strip(" |")
    value = _normalize_space(value)
    return value


def _resolve_image_tokens(text: str) -> str:
    def repl(match: re.Match[str]) -> str:
        body = _normalize_space(match.group(1) or "")
        body = re.sub(r"^Image\s+\d+\s*:?\s*", "", body)
        if _signal_chars(body) < 4:
            return ""
        return body

    value = re.sub(r"\[IMG:([^\]]+)\]", repl, str(text or ""))
    value = value.replace("](", " ")
    return _normalize_space(value)


def _signal_chars(text: str) -> int:
    return len(SIGNAL_CHAR_RE.findall(text))


def _latin_word_count(text: str) -> int:
    return len(LATIN_WORD_RE.findall(text))


def _cjk_char_count(text: str) -> int:
    return len(CJK_CHAR_RE.findall(text))


def _wrapper_ratio(text: str) -> float:
    if not text:
        return 1.0
    wrapper_chars = sum(ch in "[](){}<>*_`|" for ch in text)
    return wrapper_chars / max(len(text), 1)


def _collapse_breadcrumbs(text: str) -> str:
    if not any(sep in text for sep in (">", "›", "»")):
        return text
    parts = [_normalize_space(part) for part in re.split(r"\s*[>›»]+\s*", text) if _normalize_space(part)]
    if len(parts) < 2:
        return text
    tail = parts[-1]
    if _signal_chars(tail) >= max(8, _signal_chars(text) // 4):
        return tail
    return text


def _looks_command_like(raw: str, clean: str) -> bool:
    raw = str(raw or "").strip()
    if not raw or not clean:
        return False
    if "`" in raw or "`" in clean:
        return True
    if re.search(r"(^|\s)(?:\./|\.\./|/[\w./-]+|--[\w-]+|\w+=\S+)", raw):
        return True
    if any(token in raw for token in (" | ", " && ", " || ")):
        return True
    tokens = clean.split()
    if 1 <= len(tokens) <= 8 and any(part.startswith("-") or "/" in part or "." in part for part in tokens[1:]):
        if _latin_word_count(clean) >= 1:
            return True
    return False


def _classify_line(raw: str, clean: str) -> str:
    raw = str(raw or "").lstrip()
    if MARKDOWN_HEADING_RE.match(raw):
        return "heading"
    if raw.startswith("|") and raw.count("|") >= 2:
        return "table"
    if MARKDOWN_BULLET_RE.match(raw):
        return "bullet"
    if _looks_command_like(raw, clean):
        return "command"
    return "text"


def _table_cells(text: str) -> list[str]:
    if not text.startswith("|"):
        return []
    parts = [_normalize_space(part) for part in text.strip("|").split("|")]
    return [part for part in parts if part and not re.fullmatch(r"[:\- ]+", part)]


def _looks_weak_surface(text: str) -> bool:
    if not text:
        return True
    if text in {"?", "？"}:
        return True
    non_image = _normalize_space(re.sub(r"\[IMG:[^\]]+\]", "", text))
    if text.startswith("[IMG:") and _signal_chars(non_image) < 6:
        return True
    if text.count("[IMG:") >= 2 and _signal_chars(non_image) < 12:
        return True
    if re.fullmatch(r"[^\w\u4e00-\u9fff\u3040-\u30ff]+", text):
        return True
    if re.fullmatch(r"(?:\d+[./:-]?)+", text):
        return True
    if _signal_chars(text) <= 1:
        return True
    if _wrapper_ratio(text) >= 0.35:
        return True
    if " " not in text and len(re.findall(r"[a-z][A-Z]", text)) >= 2:
        return True
    return False


def _signal_score(kind: str, text: str) -> int:
    if _looks_weak_surface(text):
        return -100
    signal = _signal_chars(text)
    cjk = _cjk_char_count(text)
    latin_words = _latin_word_count(text)
    has_sentence = bool(re.search(r"[。！？.!?:：;；]", text))
    has_code = bool(re.search(r"`[^`]+`|(^|\s)(?:--[\w-]+|\w+=\S+|/[\w./-]+)", text))
    wrapper_ratio = _wrapper_ratio(text)

    score = 0
    if kind == "heading":
        if signal < 6:
            return -100
        score += 5
        if cjk >= 6 or latin_words >= 3:
            score += 2
        if has_sentence:
            score += 1
    elif kind == "command":
        if signal < 6:
            return -100
        score += 5
        if has_code:
            score += 2
    elif kind == "table":
        cells = _table_cells(text)
        if len(cells) < 2:
            return -100
        if sum(_signal_chars(cell) for cell in cells) < 8:
            return -100
        score += 4
        if any("`" in cell for cell in cells):
            score += 1
    elif kind == "bullet":
        if signal < 8:
            return -100
        if cjk < 6 and latin_words < 5 and not has_sentence and not has_code:
            return -100
        score += 4
        if has_sentence or cjk >= 10 or latin_words >= 8:
            score += 2
    else:
        if signal < 24:
            return -100
        if cjk < 14 and latin_words < 8 and not has_sentence and not has_code:
            return -100
        score += 4
        if has_sentence:
            score += 2

    if cjk >= 8 or latin_words >= 6:
        score += 1
    if "`" in text:
        score += 1
    if text.startswith("[IMG:") or text.count("[IMG:") >= 2:
        score -= 4
    if re.fullmatch(r"[A-Z][A-Z0-9 /&+-]{3,}", text) and latin_words <= 2:
        score -= 4
    if len(text) <= 3:
        score -= 3
    if wrapper_ratio >= 0.24:
        score -= 4
    if kind != "heading" and latin_words <= 3 and cjk == 0 and not has_sentence and len(text) <= 24:
        score -= 5
    return score


def _normalize_skeleton_text(kind: str, clean: str) -> str:
    text = clean
    if kind == "heading":
        text = re.sub(r"^#{1,6}\s*", "", text)
    elif kind == "bullet":
        text = MARKDOWN_BULLET_RE.sub("", text)
    text = _collapse_breadcrumbs(text)
    return _normalize_space(text)


def _build_skeleton_lines(rows: list[tuple[int, str]], *, limit: int = 10) -> list[str]:
    candidates: list[tuple[int, int, str, str]] = []
    seen: set[str] = set()
    for line_no, raw in rows:
        clean = _strip_markdown_links(raw)
        clean = _resolve_image_tokens(clean)
        if not clean or _looks_weak_surface(clean):
            continue
        kind = _classify_line(raw, clean)
        text = _normalize_skeleton_text(kind, clean)
        if not text:
            continue
        dedupe_key = f"{kind}:{text.casefold()}"
        if dedupe_key in seen:
            continue
        score = _signal_score(kind, text)
        if score < 2:
            continue
        seen.add(dedupe_key)
        candidates.append((score, line_no, kind, text[:160]))

    if not candidates:
        return []

    candidates = _focus_candidates(candidates)
    selected: list[str] = []
    for _, line_no, kind, text in sorted(
        sorted(candidates, key=lambda item: (-item[0], item[1]))[: max(limit * 2, limit)],
        key=lambda item: item[1],
    ):
        selected.append(f"- {kind}@L{line_no}: {text}")
        if len(selected) >= limit:
            break
    return selected


def _focus_candidates(candidates: list[tuple[int, int, str, str]]) -> list[tuple[int, int, str, str]]:
    if len(candidates) <= 3:
        return candidates
    line_numbers = [line_no for _, line_no, _, _ in candidates]
    spread = max(line_numbers) - min(line_numbers)
    window = min(max(40, spread // 3), 120)
    half_window = max(20, window // 2)

    best_score: int | None = None
    best_range: tuple[int, int] | None = None
    for _, anchor, _, _ in candidates:
        start = anchor - half_window
        end = anchor + half_window
        score = sum(item_score for item_score, line_no, _, _ in candidates if start <= line_no <= end)
        if best_score is None or score > best_score:
            best_score = score
            best_range = (start, end)

    if best_range is None:
        return candidates

    start, end = best_range
    focused = [item for item in candidates if start <= item[1] <= end]
    if len(focused) >= max(3, min(6, len(candidates) // 2)):
        return focused
    return candidates


def _iter_page_entries(log_dir: Path) -> list[PageEntry]:
    entries: list[PageEntry] = []
    for path in sorted(log_dir.glob("*.md"), reverse=True):
        try:
            text = path.read_text(encoding="utf-8")
        except Exception:
            continue
        for payload in _iter_json_blocks(text):
            entry = _parse_page_markdown(payload, source_log=path)
            if entry is not None:
                entries.append(entry)
    return entries


def _pick_entries(entries: list[PageEntry], *, limit: int) -> list[PageEntry]:
    def quality(entry: PageEntry) -> tuple[int, int, int]:
        textish = 0
        commandish = 0
        long_lines = 0
        for line in entry.skeleton_lines:
            head, _, body = line.partition(": ")
            kind = head.removeprefix("- ").split("@", 1)[0]
            if kind in {"text", "table"}:
                textish += 2
            elif kind == "command":
                commandish += 2
            elif kind == "bullet" and len(body) >= 48:
                textish += 1
            if len(body) >= 72:
                long_lines += 1
        return (textish + commandish, long_lines, len(entry.skeleton_lines))

    picked: list[PageEntry] = []
    seen_urls: set[str] = set()
    seen_hosts: dict[str, int] = {}
    ranked = sorted(entries, key=lambda item: (quality(item), item.source_log.name), reverse=True)
    for entry in ranked:
        if entry.url in seen_urls:
            continue
        if len(entry.skeleton_lines) < 4:
            continue
        contentish, long_lines, _ = quality(entry)
        if contentish < 3:
            continue
        if long_lines == 0 and not any(line.startswith("- command@") for line in entry.skeleton_lines):
            continue
        host_count = seen_hosts.get(entry.host, 0)
        if host_count >= 2:
            continue
        seen_urls.add(entry.url)
        seen_hosts[entry.host] = host_count + 1
        picked.append(entry)
        if len(picked) >= limit:
            break
    return picked


def _render(entries: list[PageEntry]) -> str:
    lines = ["# Page Skeleton Samples", ""]
    for index, entry in enumerate(entries, start=1):
        lines.append(f"## {index}. {entry.title_core}")
        lines.append(f"- url: {entry.url}")
        lines.append(f"- host: {entry.host}")
        lines.append(f"- mode: {entry.mode}")
        lines.append(f"- source_log: {entry.source_log}")
        lines.append(f"- raw_title: {entry.raw_title or '(none)'}")
        lines.append(f"- title_core: {entry.title_core}")
        lines.append(f"- path_label: {entry.path_label}")
        lines.append(f"- matched_lines: {len(entry.matched_lines)}")
        lines.append("- skeleton:")
        lines.extend(entry.skeleton_lines)
        lines.append("")
    return "\n".join(lines).rstrip() + "\n"


def main() -> int:
    parser = argparse.ArgumentParser(
        description="Extract and clean page skeleton samples from local hyw logs.",
    )
    parser.add_argument(
        "--log-dir",
        type=Path,
        default=Path("~/.hyw/logs").expanduser(),
        help="Directory containing markdown logs.",
    )
    parser.add_argument(
        "--limit",
        type=int,
        default=10,
        help="Number of samples to emit.",
    )
    parser.add_argument(
        "--output",
        type=Path,
        default=None,
        help="Optional markdown output path. Prints to stdout when omitted.",
    )
    args = parser.parse_args()

    entries = _iter_page_entries(args.log_dir)
    entries = _pick_entries(entries, limit=max(1, int(args.limit)))
    rendered = _render(entries)
    if args.output is not None:
        args.output.write_text(rendered, encoding="utf-8")
    else:
        print(rendered, end="")
    return 0


if __name__ == "__main__":
    raise SystemExit(main())



from __future__ import annotations

import os
from typing import Any

from .config import cfg_get

_JINA_CONFIG_RESERVED_KEYS = {"headers", "search", "page_extract", "prefer_free"}
_JINA_CAPABILITY_RESERVED_KEYS = {"headers", "prefer_free"}


def load_httpx_module() -> Any:
    try:
        import httpx

        return httpx
    except Exception as exc:
        raise RuntimeError("Jina AI support requires `httpx`; reinstall `hyw`.") from exc


def _stringify_header_value(value: Any) -> str:
    if isinstance(value, bool):
        return "true" if value else "false"
    return str(value).strip()


def _normalize_header_map(value: Any) -> dict[str, str]:
    if not isinstance(value, dict):
        return {}
    headers: dict[str, str] = {}
    for raw_key, raw_value in value.items():
        key = str(raw_key or "").strip()
        if not key or raw_value is None:
            continue
        text = _stringify_header_value(raw_value)
        if text:
            headers[key] = text
    return headers


def _collect_header_block(value: Any, *, reserved_keys: set[str]) -> dict[str, str]:
    if not isinstance(value, dict):
        return {}
    headers = _normalize_header_map(
        {key: val for key, val in value.items() if str(key) not in reserved_keys}
    )
    nested_headers = value.get("headers")
    if isinstance(nested_headers, dict):
        headers.update(_normalize_header_map(nested_headers))
    return headers


def _to_int(value: Any) -> int:
    try:
        return max(0, int(str(value).strip()))
    except Exception:
        return 0


def _jina_config_roots(config: dict[str, Any] | None, provider_name: str) -> list[dict[str, Any]]:
    roots = [
        cfg_get(config or {}, provider_name, {}),
        cfg_get(config or {}, f"tools.config.{provider_name}", {}),
    ]
    return [root for root in roots if isinstance(root, dict)]


def _get_header_key(headers: dict[str, str], name: str) -> str | None:
    target = str(name or "").strip().lower()
    for key in headers:
        if str(key).strip().lower() == target:
            return key
    return None


def has_auth_header(headers: dict[str, str]) -> bool:
    return _get_header_key(headers, "Authorization") is not None


def jina_headers(
    *,
    config: dict[str, Any] | None = None,
    provider_name: str,
    capability: str,
    default_accept: str,
    default_headers: dict[str, str] | None = None,
) -> dict[str, str]:
    headers = {
        "User-Agent": "hyw/0.1 (+https://github.com/kumoSleeping/heuristic_yield_websearch)",
    }
    api_key = str(os.environ.get("JINA_API_KEY") or "").strip()
    if api_key:
        headers["Authorization"] = f"Bearer {api_key}"

    for root in _jina_config_roots(config, provider_name):
        headers.update(_collect_header_block(root, reserved_keys=_JINA_CONFIG_RESERVED_KEYS))
        capability_block = root.get(capability) if isinstance(root, dict) else None
        headers.update(
            _collect_header_block(
                capability_block,
                reserved_keys=_JINA_CAPABILITY_RESERVED_KEYS,
            )
        )

    headers.setdefault("Accept", default_accept)
    for key, value in (default_headers or {}).items():
        text = str(value or "").strip()
        if text:
            headers.setdefault(str(key).strip(), text)
    return headers


async def _count_tokens_via_segment(client: Any, content: str) -> int:
    text = str(content or "")
    if not text:
        return 0
    try:
        response = await client.post(
            "https://segment.jina.ai/",
            json={"content": text},
            headers={"Accept": "application/json"},
        )
        response.raise_for_status()
        payload = response.json()
    except Exception:
        return 0

    if not isinstance(payload, dict):
        return 0
    return _to_int(payload.get("num_tokens"))


async def build_usage_meta(
    client: Any,
    *,
    response: Any,
    raw_text: str,
    provider: str,
    capability: str,
    billed: bool | None = None,
) -> dict[str, Any]:
    tokens = _to_int(response.headers.get("x-usage-tokens"))
    source = "header" if tokens > 0 else ""
    if tokens <= 0 and raw_text:
        tokens = await _count_tokens_via_segment(client, raw_text)
        if tokens > 0:
            source = "segment"

    return {
        "provider": str(provider or "").strip(),
        "capability": capability,
        "billing": {
            "mode": "paid" if billed else "free",
            "cost_usd": 0.0 if billed is False else None,
        },
        "usage": {
            "requests": 1,
            "tokens": tokens,
            "source": source or "unknown",
        },
    }


__all__ = [
    "build_usage_meta",
    "has_auth_header",
    "jina_headers",
    "load_httpx_module",
]



from __future__ import annotations

import importlib
import tempfile
import unittest
from pathlib import Path

main = importlib.import_module("core.main")
prompt = importlib.import_module("core.prompt")


class CompactContextTests(unittest.TestCase):
    def test_build_compact_context_preserves_last_menu_for_choice_reply(self) -> None:
        menu = (
            "1. 苹果\n"
            "2. 香蕉\n"
            "3. 梨\n"
            + ("补充说明 " * 40)
            + "TAIL_TOKEN"
        )
        memory = [
            {"role": "user", "content": "选一个"},
            {"role": "assistant", "content": menu},
        ]

        context = main.build_compact_context(memory, current_user_text="2")

        self.assertIsNotNone(context)
        assert context is not None
        self.assertIn("Assistant (full because current user reply looks like an option selection):", context)
        self.assertIn("TAIL_TOKEN", context)

    def test_build_compact_context_trims_last_menu_for_normal_followup(self) -> None:
        menu = (
            "1. 苹果\n"
            "2. 香蕉\n"
            "3. 梨\n"
            + ("补充说明 " * 40)
            + "TAIL_TOKEN"
        )
        memory = [
            {"role": "user", "content": "选一个"},
            {"role": "assistant", "content": menu},
        ]

        context = main.build_compact_context(memory, current_user_text="详细说说第二个")

        self.assertIsNotNone(context)
        assert context is not None
        self.assertNotIn("Assistant (full because current user reply looks like an option selection):", context)
        self.assertNotIn("TAIL_TOKEN", context)
        self.assertIn("Assistant:", context)

    def test_search_payload_markdown_uses_line_based_search_document(self) -> None:
        rendered = main._build_search_payload_markdown(
            query="entari 0.17",
            search_filters={"kl": "us-en", "df": "2026-03-13..2026-03-26", "ia": "news"},
            public_results=[
                {
                    "title": "Release Entari 0.17.4",
                    "domain": "github.com",
                    "snippet": "Fix directory-style plugin config lookup.",
                    "matched_queries": ["entari 0.17"],
                }
            ],
            skipped_duplicate=False,
        )

        self.assertIn("Filters: kl=us-en, df=2026-03-13..2026-03-26", rendered)
        self.assertNotIn("ia=", rendered)
        self.assertIn("Result lines:", rendered)
        self.assertIn(
            "L01 | [1] Release Entari 0.17.4 | github.com | matched=entari 0.17 | Fix directory-style plugin config lookup.",
            rendered,
        )

    def test_navigate_search_builds_openable_result_lines_and_ref_resolution(self) -> None:
        runtime_state = main._SessionRuntimeState()

        original_search_backend = main._search_backend

        def _fake_search_backend(query, **kwargs):
            del kwargs
            self.assertEqual(query, "entari 0.17")
            return {
                "ok": True,
                "provider": "ddgs",
                "results": [
                    {
                        "title": "Release Entari 0.17.4",
                        "url": "https://github.com/ArcletProject/Entari/releases/tag/v0.17.4",
                        "snippet": "Fix directory-style plugin config lookup.",
                        "domain": "github.com",
                    }
                ],
            }

        main._search_backend = _fake_search_backend
        try:
            payload = main._run_navigate(
                url="",
                search="entari 0.17",
                search_filters={"df": "2026-03-13..2026-03-26"},
                cfg={},
                runtime_state=runtime_state,
            )
        finally:
            main._search_backend = original_search_backend

        self.assertTrue(payload["ok"])
        self.assertIn(
            "[Release Entari 0.17.4](https://github.com/ArcletProject/Entari/releases/tag/v0.17.4)",
            payload["_raw_lines"][0],
        )

        phase_state = main._PhaseRuntimeState()
        page_id = main._set_active_page_item(phase_state, payload, round_no=1, call_id="r1c1")
        calls = [{"name": "navigate", "args": {"ref": f"{page_id}:1"}}]
        main._resolve_page_call_refs(calls, phase_state)

        self.assertEqual(
            calls[0]["args"].get("url"),
            "https://github.com/ArcletProject/Entari/releases/tag/v0.17.4",
        )

    def test_resolve_page_call_refs_supports_memory_page_markdown_links(self) -> None:
        phase_state = main._PhaseRuntimeState()
        item = main._add_context_item(
            phase_state,
            "memory.page",
            {
                "text": (
                    "L02 | [2] [记忆中的彩叶](https://tieba.baidu.com/p/10574520448) | tieba.baidu.com\n"
                    "L04 | [4] [御宅族彩叶](https://tieba.baidu.com/p/10576080529) | tieba.baidu.com"
                )
            },
        )

        calls = [
            {"name": "navigate", "args": {"ref": f"{item.item_id}:2"}},
            {"name": "navigate", "args": {"ref": f"{item.item_id}:4"}},
        ]
        main._resolve_page_call_refs(calls, phase_state)

        self.assertEqual(calls[0]["args"].get("url"), "https://tieba.baidu.com/p/10574520448")
        self.assertEqual(calls[1]["args"].get("url"), "https://tieba.baidu.com/p/10576080529")

    def test_execute_tool_payload_navigate_supports_runtime_ref_targets(self) -> None:
        runtime_state = main._SessionRuntimeState()
        runtime_state.ref_targets["1:2"] = "https://example.com/resolved"

        original_load_page_payload = main._load_page_payload

        def _fake_load_page_payload(*, url, cfg, runtime_state, progress_callback):
            del cfg, runtime_state, progress_callback
            self.assertEqual(url, "https://example.com/resolved")
            return (
                {
                    "ok": True,
                    "provider": "jina_ai",
                    "url": url,
                    "title": "Resolved",
                    "content": "alpha\nbeta",
                },
                False,
            )

        main._load_page_payload = _fake_load_page_payload
        try:
            payload = main.execute_tool_payload(
                "navigate",
                {"ref": "1:2"},
                config={},
                runtime_state=runtime_state,
            )
        finally:
            main._load_page_payload = original_load_page_payload

        self.assertTrue(payload["ok"])
        self.assertEqual(payload["url"], "https://example.com/resolved")

    def test_navigate_search_reuses_runtime_cached_results_without_refetch(self) -> None:
        runtime_state = main._SessionRuntimeState()
        runtime_state.search_results_deduped = [
            {
                "title": "Release Entari 0.17.4",
                "url": "https://github.com/ArcletProject/Entari/releases/tag/v0.17.4",
                "snippet": "Fix directory-style plugin config lookup.",
                "provider": "ddgs",
                "domain": "github.com",
                "matched_queries": ["entari 0.17 [df=2026-03-13..2026-03-26]"],
                "_order": 0,
            }
        ]

        original_search_backend = main._search_backend

        def _unexpected_search_backend(*args, **kwargs):
            raise AssertionError("cached navigate(search=...) should not refetch")

        main._search_backend = _unexpected_search_backend
        try:
            payload = main._run_navigate(
                url="",
                search="entari 0.17",
                search_filters={"df": "2026-03-13..2026-03-26"},
                cfg={},
                runtime_state=runtime_state,
            )
        finally:
            main._search_backend = original_search_backend

        self.assertTrue(payload["ok"])
        self.assertTrue(payload["from_cache"])

    def test_context_ref_supports_page_local_link_refs(self) -> None:
        self.assertEqual(main._normalize_context_ref("44:2"), "44:2")
        self.assertEqual(main._parse_context_ref("44:2"), (44, 2))
        self.assertEqual(main._parse_context_ref("0:2"), (0, 2))

    def test_default_max_rounds_is_20(self) -> None:
        self.assertEqual(main._resolve_max_rounds({}), 20)

    def test_late_round_warning_is_injected_into_system_and_user_messages(self) -> None:
        state = main._PhaseRuntimeState()
        history = [{"role": "user", "content": "查一下 OpenAI 最新模型"}]

        early_msgs = main._build_loop_messages(
            cfg={},
            prompt_text="查一下 OpenAI 最新模型",
            history=history,
            state=state,
            round_i=14,
            context=None,
        )
        late_msgs = main._build_loop_messages(
            cfg={},
            prompt_text="查一下 OpenAI 最新模型",
            history=history,
            state=state,
            round_i=15,
            context=None,
        )

        early_text = "\n".join(str(item.get("content") or "") for item in early_msgs if isinstance(item, dict))
        late_system = next(str(item.get("content") or "") for item in late_msgs if item.get("role") == "system")
        late_user_text = "\n".join(
            str(item.get("content") or "")
            for item in late_msgs
            if isinstance(item, dict) and item.get("role") == "user"
        )

        self.assertNotIn(main.LATE_ROUND_FINAL_REPLY_PROMPT, early_text)
        self.assertIn(main.LATE_ROUND_FINAL_REPLY_PROMPT, late_system)
        self.assertIn(main.LATE_ROUND_FINAL_REPLY_PROMPT, late_user_text)

    def test_first_search_prompt_includes_formatted_user_message(self) -> None:
        state = main._PhaseRuntimeState()
        history = [{"role": "user", "content": "告诉我最近发布的 OpenAI 模型有哪些"}]

        msgs = main._build_loop_messages(
            cfg={},
            prompt_text="告诉我最近发布的 OpenAI 模型有哪些",
            history=history,
            state=state,
            round_i=0,
            context=None,
        )

        user_text = "\n".join(
            str(item.get("content") or "")
            for item in msgs
            if isinstance(item, dict) and item.get("role") == "user"
        )

        self.assertIn("告诉我最近发布的 OpenAI 模型有哪些", user_text)
        self.assertNotIn("{user_message}", user_text)

    def test_first_search_prompt_is_skipped_when_prior_turn_summary_exists(self) -> None:
        state = main._PhaseRuntimeState()
        memory = [
            {"role": "user", "content": "葱喵Bot"},
            {"role": "assistant", "content": "它是一个待查询对象，请补充你的问题。"},
        ]
        context = main.build_compact_context(memory, current_user_text="name在配置里怎么配置")
        history = [{"role": "user", "content": "name在配置里怎么配置"}]

        msgs = main._build_loop_messages(
            cfg={},
            prompt_text="name在配置里怎么配置",
            history=history,
            state=state,
            round_i=0,
            context=context,
        )

        user_text = "\n".join(
            str(item.get("content") or "")
            for item in msgs
            if isinstance(item, dict) and item.get("role") == "user"
        )

        self.assertNotIn("首轮搜索严格参考", user_text)
        self.assertNotIn("默认认为任何孤立名词", user_text)

    def test_post_search_prompt_no_longer_repeats_latest_query_guardrail(self) -> None:
        state = main._PhaseRuntimeState()
        state.disclosure_step = 1
        state.disclosure_refine_consumed = False
        history = [{"role": "user", "content": "告诉我最近发布的 OpenAI 模型有哪些"}]

        msgs = main._build_loop_messages(
            cfg={},
            prompt_text="告诉我最近发布的 OpenAI 模型有哪些",
            history=history,
            state=state,
            round_i=1,
            context=None,
        )

        user_text = "\n".join(
            str(item.get("content") or "")
            for item in msgs
            if isinstance(item, dict) and item.get("role") == "user"
        )

        self.assertNotIn("先建立官方时间轴，再下结论", user_text)

    def test_tool_validation_currently_relies_on_prompt_control(self) -> None:
        state = main._PhaseRuntimeState()
        payload = {
            "url": "https://example.com/start",
            "title": "Demo",
            "from_cache": False,
            "_raw_lines": ["hello"],
            "count": 1,
        }
        main._set_active_page_item(state, payload, round_no=1, call_id="r1c1")

        result = main._validate_progressive_tool_requirements(
            state,
            [{"name": "navigate", "args": {"url": "https://example.com/next"}}],
        )
        self.assertEqual(result, "")

    def test_multiple_open_pages_remain_outstanding_and_visible(self) -> None:
        state = main._PhaseRuntimeState()
        payload_a = {
            "url": "https://example.com/a",
            "title": "Page A",
            "from_cache": False,
            "_raw_lines": ["alpha"],
            "count": 1,
        }
        payload_b = {
            "url": "https://example.com/b",
            "title": "Page B",
            "from_cache": False,
            "_raw_lines": ["beta"],
            "count": 1,
        }

        first_id = main._set_active_page_item(state, payload_a, round_no=1, call_id="r1c1")
        second_id = main._set_active_page_item(state, payload_b, round_no=1, call_id="r1c2")

        self.assertEqual(state.active_page_ids, [first_id, second_id])
        message = main._build_active_page_message(state)
        self.assertIn(f"# Page [{first_id}]", message)
        self.assertIn(f"# Page [{second_id}]", message)

    def test_replace_active_pages_keeps_lines_and_clears_previous_pages(self) -> None:
        state = main._PhaseRuntimeState()
        payload_a = {
            "url": "https://example.com/a",
            "title": "Page A",
            "from_cache": False,
            "_raw_lines": ["alpha"],
            "count": 1,
        }
        payload_b = {
            "url": "https://example.com/b",
            "title": "Page B",
            "from_cache": False,
            "_raw_lines": ["beta"],
            "count": 1,
        }

        first_id = main._set_active_page_item(state, payload_a, round_no=1, call_id="r1c1")
        second_id = main._set_active_page_item(state, payload_b, round_no=1, call_id="r1c2")

        replaced_ids, created_ids = main._replace_active_pages(
            args={"keep": ["L1"]},
            state=state,
            round_no=2,
        )

        self.assertEqual(replaced_ids, [first_id, second_id])
        self.assertEqual(state.active_page_ids, [])
        self.assertTrue(created_ids)
        self.assertEqual(len(created_ids), 2)

    def test_round_brief_explicitly_shows_unclosed_open_count(self) -> None:
        state = main._PhaseRuntimeState()
        payload_a = {
            "url": "https://example.com/a",
            "title": "Page A",
            "from_cache": False,
            "_raw_lines": ["alpha"],
            "count": 1,
        }
        payload_b = {
            "url": "https://example.com/b",
            "title": "Page B",
            "from_cache": False,
            "_raw_lines": ["beta"],
            "count": 1,
        }

        first_id = main._set_active_page_item(state, payload_a, round_no=1, call_id="r1c1")
        second_id = main._set_active_page_item(state, payload_b, round_no=1, call_id="r1c2")
        brief = main._build_round_brief_message(state, round_i=1, user_message="test")

        expected = prompt.ACTIVE_PAGE_STATE_PROMPT.format(count=2, ids=f"{first_id}, {second_id}")
        self.assertIn(expected, brief)

    def test_log_model_call_uses_new_markdown_structure_without_tool_sections(self) -> None:
        with tempfile.TemporaryDirectory() as tmpdir:
            log_id = "session.md"
            config = {"log_dir": tmpdir}
            messages = [
                {"role": "system", "content": "You are hyw."},
                {"role": "user", "content": "最近发布的 openai 模型"},
            ]
            tool_calls = [{"name": "navigate", "args": {"search": "OpenAI latest models"}}]

            main.log_model_call(
                label="round 1",
                model="demo-model",
                messages=messages,
                output="这是整理后的回答。",
                tool_calls=tool_calls,
                turn=1,
                title="最近发布的 openai 模型",
                usage={"prompt_tokens": 10, "completion_tokens": 20, "total_tokens": 30},
                cost=0.123456,
                duration_ms=789.0,
                config=config,
                log_id=log_id,
            )
            main.log_tool_selection(
                round_i=0,
                calls=tool_calls,
                config=config,
                log_id=log_id,
            )
            main.log_tool_call(
                name="navigate",
                args={"search": "OpenAI latest models"},
                payload={"ok": True, "results": [{"title": "Official"}]},
                duration_ms=456.0,
                turn=1,
                config=config,
                log_id=log_id,
            )

            content = (Path(tmpdir) / log_id).read_text(encoding="utf-8")

        self.assertIn("# 最近发布的 openai 模型", content)
        self.assertIn("## Turn: 1", content)
        self.assertIn("### 模型Input", content)
        self.assertIn("### 模型Output", content)
        self.assertIn("### 模型工具Output", content)
        self.assertIn("### 模型时间、开销", content)
        self.assertIn("### 工具执行摘要", content)
        self.assertIn('"name": "navigate"', content)
        self.assertIn('"tool": "navigate"', content)
        self.assertIn("这是整理后的回答。", content)
        self.assertNotIn("selected tools", content)
        self.assertNotIn("### Args", content)
        self.assertNotIn("### Result", content)
        self.assertNotIn("tool `navigate`", content)

    def test_extract_raw_tool_calls_preserves_provider_id_without_injecting_call_id(self) -> None:
        payload = {
            "tool_calls": [
                {
                    "id": "call_demo",
                    "function": {
                        "name": "navigate",
                        "arguments": "{\"ref\":\"4:2\",\"keep\":[\"L12-L18\"]}",
                    },
                }
            ]
        }

        rows = main._extract_raw_tool_calls(payload)

        self.assertEqual(
            rows,
            [{"id": "call_demo", "name": "navigate", "args": {"ref": "4:2", "keep": ["L12-L18"]}}],
        )

    def test_navigate_requires_valid_keep_and_rejects_old_aliases(self) -> None:
        self.assertIsNone(main._sanitize_native_tool_call("navigate", {"ref": "4:2"}))
        self.assertIsNone(main._sanitize_native_tool_call("navigate", {"keep": ["L1"]}))
        self.assertIsNone(main._sanitize_native_tool_call("navigate", {"ref": "4:2", "keep": ["bad"]}))
        self.assertIsNone(main._sanitize_native_tool_call("open", {"ref": "4:2", "keep": ["L1"]}))
        self.assertIsNone(main._sanitize_native_tool_call("close", {"ref": "4", "keep": ["L1"]}))
        self.assertEqual(
            main._sanitize_native_tool_call("navigate", {"ref": "4:2", "keep": "L0"}),
            {"name": "navigate", "args": {"ref": "4:2", "keep": "L0"}},
        )
        self.assertEqual(
            main._sanitize_native_tool_call("navigate", {"ref": "4:2", "keep": ["L12-L18"]}),
            {"name": "navigate", "args": {"ref": "4:2", "keep": ["L12-L18"]}},
        )


if __name__ == "__main__":
    unittest.main()



from __future__ import annotations

import asyncio
import re
from typing import Any, Dict, List, Optional
from urllib.parse import parse_qs, urlencode, unquote, urlsplit

from .jina_common import (
    build_usage_meta,
    has_auth_header,
    jina_headers,
    load_httpx_module,
)

_DDG_HTML_SEARCH_URL = "https://html.duckduckgo.com/html/"
_JINA_DDGS_DEFAULT_HEADERS = {
    "X-Engine": "browser",
    "X-Return-Format": "markdown",
}


def _normalize_region(kl: Optional[str]) -> str:
    raw = str(kl or "").strip().lower()
    return raw or "wt-wt"


def _normalize_search_filters(
    *,
    kl: Any = None,
    region: Any = None,
    df: Any = None,
    timelimit: Any = None,
    t: Any = None,
    ia: Any = None,
) -> dict[str, str]:
    filters: dict[str, str] = {}

    region_text = str(kl or region or "").strip().lower()
    if region_text:
        filters["kl"] = region_text

    time_text = str(df or timelimit or "").strip()
    if time_text:
        filters["df"] = time_text

    source_tag = str(t or "").strip()
    if source_tag:
        filters["t"] = source_tag

    interface = str(ia or "").strip().lower()
    if interface:
        filters["ia"] = interface

    return filters


def build_duckduckgo_html_url(
    query: str,
    *,
    kl: Any = None,
    region: Any = None,
    df: Any = None,
    timelimit: Any = None,
    t: Any = None,
    ia: Any = None,
) -> str:
    clean_query = str(query or "").strip()
    if not clean_query:
        raise ValueError("query is empty")

    filters = _normalize_search_filters(
        kl=kl,
        region=region,
        df=df,
        timelimit=timelimit,
        t=t,
        ia=ia,
    )
    params: list[tuple[str, str]] = [("q", clean_query)]
    if filters.get("kl"):
        params.append(("l", filters["kl"]))
    if filters.get("df"):
        params.append(("df", filters["df"]))
    if filters.get("t"):
        params.append(("t", filters["t"]))
    params.append(("ia", filters.get("ia") or "web"))
    return f"{_DDG_HTML_SEARCH_URL}?{urlencode(params)}"


def _build_ddgs_kwargs(
    *,
    kl: Any = None,
    region: Any = None,
    df: Any = None,
    timelimit: Any = None,
    max_results: int = 5,
) -> dict[str, Any]:
    filters = _normalize_search_filters(
        kl=kl,
        region=region,
        df=df,
        timelimit=timelimit,
    )
    return {
        "region": _normalize_region(filters.get("kl")),
        "safesearch": "moderate",
        "timelimit": filters.get("df") or None,
        "max_results": max(1, int(max_results)),
        "page": 1,
        "backend": "duckduckgo",
    }


def _load_ddgs_class() -> Any:
    try:
        from ddgs import DDGS

        return DDGS
    except Exception as ddgs_exc:
        try:
            from duckduckgo_search import DDGS

            return DDGS
        except Exception:
            raise RuntimeError(
                "ddgs is not installed; reinstall `hyw` or install `ddgs` manually."
            ) from ddgs_exc


def _unwrap_duckduckgo_redirect(url: str) -> str:
    text = str(url or "").strip()
    if not text:
        return ""
    try:
        parsed = urlsplit(text)
    except Exception:
        return text

    host = parsed.netloc.lower().strip()
    if host.startswith("www."):
        host = host[4:]
    if host.endswith("duckduckgo.com") and parsed.path.startswith("/l/"):
        target = parse_qs(parsed.query).get("uddg", [""])[0]
        if target:
            return unquote(target).strip()
    return text


def _normalize_search_row(row: Dict[str, Any], *, provider: str) -> Optional[Dict[str, Any]]:
    url = _unwrap_duckduckgo_redirect(
        str(
            row.get("href")
            or row.get("url")
            or row.get("link")
            or row.get("source")
            or row.get("document_url")
            or ""
        ).strip()
    )
    if url.startswith("//"):
        url = "https:" + url
    if not url.startswith(("http://", "https://")):
        return None

    title = str(
        row.get("title")
        or row.get("name")
        or row.get("headline")
        or row.get("text")
        or ""
    ).strip() or "No Title"
    snippet = str(
        row.get("body")
        or row.get("snippet")
        or row.get("description")
        or row.get("content")
        or row.get("text")
        or ""
    ).strip()
    if snippet.startswith(title):
        snippet = snippet[len(title) :].strip(" -:\n")
    snippet = re.sub(r"\s+", " ", snippet).strip()

    return {
        "title": title,
        "url": url,
        "snippet": snippet[:500],
        "provider": provider,
    }


def _dedupe_rows(
    raw_rows: list[dict[str, Any]],
    *,
    provider: str,
    max_results: int,
) -> list[dict[str, Any]]:
    results: list[dict[str, Any]] = []
    seen: set[str] = set()
    for row in raw_rows:
        if not isinstance(row, dict):
            continue
        normalized = _normalize_search_row(row, provider=provider)
        if not normalized:
            continue
        url = normalized["url"]
        if url in seen:
            continue
        seen.add(url)
        results.append(normalized)
        if len(results) >= max(1, int(max_results)):
            break
    return results


def _clean_markdown_line(text: str) -> str:
    clean = str(text or "").strip()
    if not clean:
        return ""
    clean = re.sub(r"!\[[^\]]*\]\([^)]+\)", "", clean)
    clean = re.sub(r"\[([^\]]+)\]\((https?://[^)]+)\)", r"\1", clean)
    clean = re.sub(r"[*_`#>]+", " ", clean)
    clean = re.sub(r"\s+", " ", clean).strip()
    return clean


def _looks_like_linkish_stub(text: str) -> bool:
    clean = str(text or "").strip()
    if not clean:
        return True
    lowered = clean.lower()
    if lowered == "duckduckgo":
        return True
    if clean.startswith(("http://", "https://")):
        return True
    if " " not in clean and ("/" in clean or "." in clean):
        return True
    return False


def _extract_markdown_snippet(text: str, *, title: str) -> str:
    parts: list[str] = []
    clean_title = str(title or "").strip()
    for raw_line in str(text or "").splitlines():
        clean = _clean_markdown_line(raw_line)
        if not clean:
            continue
        if clean_title and clean == clean_title:
            continue
        if _looks_like_linkish_stub(clean):
            continue
        parts.append(clean)
    return re.sub(r"\s+", " ", " ".join(parts)).strip()[:500]


def _parse_markdown_links(text: str, *, provider: str) -> list[dict[str, Any]]:
    rows: list[dict[str, Any]] = []
    seen: set[str] = set()
    link_re = re.compile(r"\[(?P<title>[^\]]+)\]\((?P<url>https?://[^)]+)\)")
    for match in link_re.finditer(str(text or "")):
        title = str(match.group("title") or "").strip()
        url = _unwrap_duckduckgo_redirect(str(match.group("url") or "").strip())
        if not title or not url or url in seen:
            continue
        if title.lower().startswith("image "):
            continue
        seen.add(url)
        rows.append({"title": title, "url": url, "snippet": "", "provider": provider})
    return rows


def _parse_jina_ddgs_markdown(text: str, *, max_results: int) -> list[dict[str, Any]]:
    heading_re = re.compile(
        r"^##\s+\[(?P<title>[^\]]+)\]\((?P<url>https?://[^)]+)\)\s*$",
        flags=re.MULTILINE,
    )
    matches = list(heading_re.finditer(str(text or "")))
    rows: list[dict[str, Any]] = []

    for index, match in enumerate(matches):
        body_start = match.end()
        body_end = matches[index + 1].start() if index + 1 < len(matches) else len(str(text or ""))
        title = str(match.group("title") or "").strip()
        url = _unwrap_duckduckgo_redirect(str(match.group("url") or "").strip())
        snippet = _extract_markdown_snippet(str(text or "")[body_start:body_end], title=title)
        rows.append(
            {
                "title": title or "No Title",
                "url": url,
                "snippet": snippet,
                "provider": "jina_ddgs",
            }
        )

    if not rows:
        rows = _parse_markdown_links(text, provider="jina_ddgs")
    return _dedupe_rows(rows, provider="jina_ddgs", max_results=max_results)


async def ddgs_search(
    query: str,
    kl: Optional[str] = None,
    max_results: int = 5,
    **kwargs: Any,
) -> List[Dict[str, Any]]:
    def _run() -> List[Dict[str, Any]]:
        DDGS = _load_ddgs_class()
        params = _build_ddgs_kwargs(
            kl=kl,
            region=kwargs.get("region"),
            df=kwargs.get("df"),
            timelimit=kwargs.get("timelimit"),
            max_results=max_results,
        )
        try:
            return list(DDGS().text(query, **params) or [])
        except TypeError:
            params.pop("backend", None)
            return list(DDGS().text(query, **params) or [])

    raw_rows = await asyncio.to_thread(_run)
    return _dedupe_rows(
        [row for row in raw_rows if isinstance(row, dict)],
        provider="ddgs",
        max_results=max_results,
    )


async def jina_ddgs_search(
    query: str,
    kl: Optional[str] = None,
    max_results: int = 5,
    config: dict[str, Any] | None = None,
    **kwargs: Any,
) -> Dict[str, Any]:
    httpx = load_httpx_module()
    search_url = build_duckduckgo_html_url(
        query,
        kl=kl,
        region=kwargs.get("region"),
        df=kwargs.get("df"),
        timelimit=kwargs.get("timelimit"),
        t=kwargs.get("t"),
        ia=kwargs.get("ia"),
    )
    headers = jina_headers(
        config=config,
        provider_name="jina_ddgs",
        capability="search",
        default_accept="text/plain",
        default_headers=_JINA_DDGS_DEFAULT_HEADERS,
    )
    target_url = f"https://r.jina.ai/{search_url}"

    async with httpx.AsyncClient(timeout=20.0, follow_redirects=True) as client:
        response = await client.get(target_url, headers=headers)
        response.raise_for_status()
        raw_text = response.text
        meta = await build_usage_meta(
            client,
            response=response,
            raw_text=raw_text,
            provider="jina_ddgs",
            capability="search",
            billed=has_auth_header(headers),
        )

    return {
        "results": _parse_jina_ddgs_markdown(raw_text, max_results=max_results),
        "_meta": meta,
    }


__all__ = [
    "build_duckduckgo_html_url",
    "ddgs_search",
    "jina_ddgs_search",
]



from __future__ import annotations

import json
import re
from typing import Any
from urllib.parse import urlparse

from .search_document import format_search_filters


def sanitize_tool_view_args(value: Any) -> Any:
    if isinstance(value, dict):
        cleaned: dict[str, Any] = {}
        for raw_key, raw_value in value.items():
            key = str(raw_key or "").strip()
            if not key or key.startswith("_"):
                continue
            cleaned[key] = sanitize_tool_view_args(raw_value)
        return cleaned
    if isinstance(value, list):
        return [sanitize_tool_view_args(item) for item in value]
    if isinstance(value, tuple):
        return [sanitize_tool_view_args(item) for item in value]
    return value


def _short_host(url: str) -> str:
    raw_url = str(url or "").strip()
    if not raw_url:
        return ""
    parsed = urlparse(raw_url)
    host = str(parsed.netloc or parsed.path or "").strip()
    if host.startswith("www."):
        host = host[4:]
    return host


def _short_page_target(url: str, *, max_segments: int = 4, max_chars: int = 72) -> str:
    raw_url = str(url or "").strip()
    if not raw_url:
        return ""
    parsed = urlparse(raw_url)
    host = _short_host(raw_url)
    path = str(parsed.path or "").strip("/")
    if not host:
        return _truncate_line(path, max_chars) if path else ""
    if not path:
        return host
    segments = [segment for segment in path.split("/") if segment]
    if len(segments) > max(1, int(max_segments)):
        path = "/".join(segments[: max(1, int(max_segments))]) + "/…"
    else:
        path = "/".join(segments)
    return _truncate_line(f"{host}/{path}", max_chars)


def _clean_inline_text(value: Any) -> str:
    return re.sub(r"\s+", " ", str(value or "").strip())


def _clean_inline_list(value: Any, *, limit: int = 2) -> list[str]:
    if not isinstance(value, list):
        return []
    items: list[str] = []
    seen: set[str] = set()
    for item in value:
        text = _clean_inline_text(item)
        normalized = text.casefold()
        if not text or normalized in seen:
            continue
        seen.add(normalized)
        items.append(text)
        if len(items) >= max(1, int(limit)):
            break
    return items


def _pattern_summary(value: Any) -> str:
    if isinstance(value, list):
        return " | ".join(_clean_inline_list(value, limit=8))
    text = _clean_inline_text(value)
    if not text:
        return ""
    parts = [part.strip() for part in text.split("|")]
    return " | ".join(part for part in parts if part)


def _truncate_line(text: str, max_chars: int) -> str:
    line = str(text or "").strip()
    if len(line) <= max_chars:
        return line
    return line[: max_chars - 1].rstrip() + "…"


def _window_label(value: Any) -> str:
    text = str(value or "").strip()
    if not text:
        return ""
    if text.lower() == "all":
        return "All"
    match = re.search(r"\d+", text)
    if not match:
        return ""
    return f"L{int(match.group(0))}"


def _numeric_value(*values: Any) -> int | None:
    for value in values:
        if value in (None, ""):
            continue
        try:
            return int(value)
        except Exception:
            continue
    return None


def _list_count(value: Any) -> int:
    return len(value) if isinstance(value, list) else 0


def _navigate_mode_label(payload: dict[str, Any]) -> str:
    mode = str(payload.get("mode") or "").strip().lower()
    if mode == "range":
        start_line = _numeric_value(payload.get("start_line"))
        end_line = _numeric_value(payload.get("end_line"))
        if start_line is not None and end_line is not None and start_line > 0 and end_line >= start_line:
            return f"L{start_line}-L{end_line}"
        return "range"
    if mode == "sample":
        return "sample"
    return ""


def _navigate_sample_stats(payload: dict[str, Any]) -> str:
    shown = _numeric_value(payload.get("count"))
    if shown is not None and shown > 0:
        return f"{shown} lines"
    return ""


def _navigate_title(payload: dict[str, Any], *, max_chars: int = 72) -> str:
    title = _clean_inline_text(payload.get("title"))
    if not title:
        return ""
    return _truncate_line(title, max_chars)


def _keep_text(value: Any) -> str:
    if isinstance(value, str) and value.strip():
        return value.strip()
    if isinstance(value, list):
        rendered = ", ".join(_clean_inline_text(item) for item in value if _clean_inline_text(item))
        return rendered.strip()
    return ""


def _plan_counts(payload: dict[str, Any]) -> tuple[int, int]:
    create_count = _numeric_value(payload.get("_created_count"))
    if create_count is None:
        create_count = _list_count(payload.get("create"))
    update_count = _numeric_value(payload.get("_updated_count"))
    if update_count is None:
        update_count = _list_count(payload.get("update"))
    return max(0, int(create_count or 0)), max(0, int(update_count or 0))


def format_tool_view_argument(name: str, arguments: Any, *, max_items: int = 12, max_chars: int = 160) -> str:
    payload = sanitize_tool_view_args(arguments)
    if not isinstance(payload, dict):
        return ""

    if name == "navigate":
        search = _clean_inline_text(payload.get("search"))
        keep_text = _keep_text(payload.get("keep"))
        if search:
            filter_text = format_search_filters(payload)
            target = search if not filter_text else f"{search} [{filter_text}]"
            if keep_text:
                target += f" keep {keep_text}"
            return f"\"{target}\""
        host = _short_host(str(payload.get("url") or "").strip())
        ref = str(payload.get("ref") or "").strip()
        target = host or ref
        if keep_text:
            target = f"{target} keep {keep_text}".strip() if target else f"keep {keep_text}"
        return f"\"{target}\"" if target else ""

    if name == "context_delete":
        ids = payload.get("ids")
        if isinstance(ids, list) and ids:
            return f"Delete {len(ids)}"
        return ""

    if not payload:
        return ""
    text = json.dumps(payload, ensure_ascii=False)
    if len(text) > max_chars:
        return text[: max_chars - 1].rstrip() + "…"
    return text


def format_tool_view_text(name: str, arguments: Any, *, max_chars: int = 160) -> str:
    payload = arguments if isinstance(arguments, dict) else {}
    tool_name = str(name or "").strip()

    if tool_name == "navigate":
        search = _clean_inline_text(payload.get("search"))
        keep_text = _keep_text(payload.get("keep"))
        if search:
            filter_text = format_search_filters(payload)
            line = f"Navigate Search \"{search}\""
            if filter_text:
                line += f" [{filter_text}]"
            if keep_text:
                line += f" keep {keep_text}"
            sample_stats = _navigate_sample_stats(payload)
            if sample_stats:
                line += f" · {sample_stats}"
            return _truncate_line(line, max_chars)
        target = _short_page_target(str(payload.get("url") or "").strip())
        ref = str(payload.get("ref") or "").strip()
        if not target and ref:
            target = f"#{ref}"
        sample_stats = _navigate_sample_stats(payload)
        title = _navigate_title(payload)
        line = "Navigate"
        if sample_stats:
            line += f" {sample_stats}"
        if title:
            line += f" \"{title}\""
        elif target:
            line += f" in \"{target}\""
        if keep_text:
            line += f" keep {keep_text}"
        return _truncate_line(line, max_chars)

    if tool_name == "context_delete":
        count = _numeric_value(payload.get("_count"), payload.get("count"))
        if count is None:
            ids = payload.get("ids")
            if isinstance(ids, list) and ids:
                count = len(ids)
        if count is not None:
            return f"Context: Delete {count}"
        return "Context: Delete"

    fallback = format_tool_view_argument(tool_name, payload, max_chars=max_chars)
    if fallback:
        return _truncate_line(f"{tool_name} {fallback}", max_chars)
    return _truncate_line(tool_name, max_chars)



from __future__ import annotations

import importlib
import unittest

codex_transport = importlib.import_module("core.codex_transport")


class CodexTransportRequestTests(unittest.TestCase):
    def test_build_request_body_maps_fast_service_tier_to_priority(self) -> None:
        body = codex_transport.build_request_body(
            cfg={
                "model": "gpt-5.4",
                "reasoning_effort": "xhigh",
                "service_tier": "fast",
            },
            messages=[{"role": "user", "content": "hello"}],
            tools=[],
            session_id="test-session",
            stream=True,
        )

        self.assertEqual(body.get("service_tier"), "priority")
        self.assertEqual(body.get("reasoning"), {"effort": "xhigh"})

    def test_build_request_body_uses_plain_messages_for_text_only_turns(self) -> None:
        body = codex_transport.build_request_body(
            cfg={"model": "gpt-5.4"},
            messages=[
                {"role": "system", "content": "Be brief."},
                {"role": "user", "content": "hello"},
            ],
            tools=[],
            session_id="test-session",
            stream=False,
        )

        self.assertEqual(body.get("instructions"), "Be brief.")
        self.assertEqual(body.get("input"), [{"role": "user", "content": "hello"}])


if __name__ == "__main__":
    unittest.main()
